import pygame
import os

class ChessColor:
    def __init__(self, light, dark):
        self.light = light
        self.dark = dark

class ChessConst:
    screenWidth = 800
    screenHeight = 800
    boardRows = 8
    boardCols = 8
    squareSize = screenWidth // boardCols

class ChessMove:
    def __init__(self, start, end):
        self.start = start
        self.end = end

    def __str__(self):
        moveStr = ''
        moveStr += f'({self.start.col}, {self.start.row})'
        moveStr += f' -> ({self.end.col}, {self.end.row})'
        return moveStr

    def __eq__(self, other):
        return self.start == other.start and self.end == other.end

class ChessTheme:
    def __init__(self, lightBg, darkBg, 
                       lightTrace, darkTrace,
                       lightMoves, darkMoves):
        
        self.bg = ChessColor(lightBg, darkBg)
        self.trace = ChessColor(lightTrace, darkTrace)
        self.moveList = ChessColor(lightMoves, darkMoves)

class ChessSound:
    def __init__(self, path):
        self.path = path
        self.sound = pygame.mixer.Sound(path)
    
    def play(self):
        pygame.mixer.Sound.play(self.sound)

class ChessConfig:
    def __init__(self):
        self.themeList = []
        self.addThemes()
        self.themeIndex = 0
        self.theme = self.themeList[self.themeIndex]
        self.font = pygame.font.SysFont('monospace', 18, bold=True)
        self.moveSound = ChessSound(
            os.path.join('assets/sounds/move.wav'))
        self.captureSound = ChessSound(
            os.path.join('assets/sounds/capture.wav'))

    def changeTheme(self):
        self.themeIndex += 1
        self.themeIndex %= len(self.themeList)
        self.theme = self.themeList[self.themeIndex]

    def addThemes(self):
        greenTheme = ChessTheme((234, 235, 200), (119, 154, 88), (244, 247, 116), (172, 195, 51), '#C86464', '#C84646')
        brownTheme = ChessTheme((235, 209, 166), (165, 117, 80), (245, 234, 100), (209, 185, 59), '#C86464', '#C84646')
        blueTheme = ChessTheme((229, 228, 200), (60, 95, 135), (123, 187, 227), (43, 119, 191), '#C86464', '#C84646')
        grayTheme = ChessTheme((120, 119, 118), (86, 85, 84), (99, 126, 143), (82, 102, 128), '#C86464', '#C84646')

        self.themeList = [greenTheme, brownTheme, blueTheme, grayTheme]

class ChessDragger:
    def __init__(self):
        self.piece = None
        self.isDragging = False
        self.mouseX = 0
        self.mouseY = 0
        self.startRow = 0
        self.startCol = 0

    def updateBlit(self, surface):
        self.piece.setTexture(size=80)
        texture = self.piece.texture
        img = pygame.image.load(texture)
        imgCenter = (self.mouseX, self.mouseY)
        self.piece.textureRect = img.get_rect(center=imgCenter)
        surface.blit(img, self.piece.textureRect)
    
    def updateMousePosition(self, position):
        self.mouseX, self.mouseY = position

    def saveInitialPosition(self, position):
        self.startRow = position[1] // ChessConst.squareSize
        self.startCol = position[0] // ChessConst.squareSize

    def startDraggingPiece(self, piece):
        self.piece = piece
        self.isDragging = True

    def stopDraggingPiece(self):
        self.piece = None
        self.isDragging = False

class ChessSquare:
    ALPHACOLS = {0: 'a', 1: 'b', 2: 'c', 3: 'd', 4: 'e', 5: 'f', 6: 'g', 7: 'h'}

    def __init__(self, row, col, piece=None):
        self.row = row
        self.col = col
        self.piece = piece
        self.alphacol = self.ALPHACOLS[col]

    def __eq__(self, other):
        return self.row == other.row and self.col == other.col

    def hasPiece(self):
        return self.piece is not None

    def isEmpty(self):
        return not self.hasPiece()

    def hasTeamPiece(self, color):
        return self.hasPiece() and self.piece.color == color

    def hasOpponentPiece(self, color):
        return self.hasPiece() and self.piece.color != color

    def isEmptyOrOpponent(self, color):
        return self.isEmpty() or self.hasOpponentPiece(color)

    @staticmethod
    def isInRange(*args):
        for arg in args:
            if arg < 0 or arg > 7:
                return False
        
        return True

    @staticmethod
    def getAlphacol(col):
        ALPHACOLS = {0: 'a', 1: 'b', 2: 'c', 3: 'd', 4: 'e', 5: 'f', 6: 'g', 7: 'h'}
        return ALPHACOLS[col]

